#ifndef MAPEDITOR_H
#define MAPEDITOR_H

#include <QBuffer>
#include <QGraphicsView>

#include "mapcenterobj.h"
#include "file.h"
#include "misc.h"
#include "parser.h"
#include "selection.h"
#include "settings.h"


class MapEditor : public QGraphicsView, public xmlObj {
    Q_OBJECT

public:
    MapEditor(QWidget* parent=0);
	~MapEditor();
    MapCenterObj* getMapCenter();
	QGraphicsScene * getScene();
	bool isRepositionBlocked(); // block while load or undo
	
private:
	QString getName(const LinkableMapObj*);	// Get e.g. heading or filename
	void makeTmpDirs();		// create temporary directories
    QString saveToDir(const QString&,const QString &,bool, const QPointF &,LinkableMapObj*);
    void saveStateChangingPart (LinkableMapObj *, LinkableMapObj *, const QString &, const QString &);
    void saveStateRemovingPart (LinkableMapObj *, const QString &);
    void saveState(LinkableMapObj *, const QString &, LinkableMapObj *, const QString &, const QString &);
    void saveState(const QString &, const QString &, const QString &, const QString &, const QString &);
    void saveState(const SaveMode&, const QString &, const QString &, const QString &, const QString &, const QString &, LinkableMapObj *);
public:	
    void parseAtom (const QString &);	
	void runScript (QString);
private:
    void addFloatImageInt(const QPixmap &img);

public:
	bool isDefault();		// false, if map was changed once
    bool isUnsaved();		// save necessary
    bool hasChanged();		// undo possible
	void setChanged();		// called from NoteEditor via LMO
	void closeMap();
	void setFilePath (QString);	
	void setFilePath (QString,QString);	
	QString getFilePath ();	// Full path e.g. "/home/tux/map.xml"
	QString getFileName ();	// e.g. "map.xml"
	QString getMapName ();	// e.g. "map"
	QString getDestPath (); // e.g. "/home/tux/map.vym"
    ErrorCode load (QString, LoadMode );	// newmap, import/replace selection
public:
    int save(const SaveMode &);	// Save map 
	void setZipped(bool);		// save map zipped
	bool saveZipped();			// 1 if file will be saved zipped
    void print();				// print canvas
	void setAntiAlias (bool);
	void setSmoothPixmap(bool);
private:
    QPixmap getPixmap();
	void setHideTmpMode (HideTmpMode);	// temporary hide stuff
	HideTmpMode getHideTmpMode();		// temporary hide stuff
public:
	void setExportMode (bool);			// temporary hide stuff during export
    void exportImage (QString fn);			// export as PNG	
    void exportImage (QString fn, QString);	// export in given format
    void exportOOPresentation(const QString &,const QString &);
    void exportXML(const QString&);		// export to directory
    void clear();		// clear map
    void copy();		// copy branch to clipboard
    void redo();		// redo last action
	bool isRedoAvailable();
    void undo();		// undo last action
	bool isUndoAvailable();
	void gotoHistoryStep (int);// goto a step in history
private:	
    void addMapReplaceInt(const QString & undoSel, const QString & path);
    void addMapInsertInt (const QString & path, int pos);
    void pasteNoSave();		// paste clipboard to branch
    void cutNoSave();	// cut to clipboard
public:	
    void paste();		// paste clipboard to branch and backup
    void cut();			// cut to clipboard and backup
    void move    (const int &x, const int &y);
    void moveRel (const int &x, const int &y);
    void moveBranchUp();
    void moveBranchDown();
private:	
	void linkTo(const QString &);
public:	
	QString getHeading (bool &ok,QPoint &p); // Get heading, ok if selection is branch
	void setHeading(const QString &);
private:
	void setHeadingInt(const QString &);
	void setURLInt(const QString &);		// Just set the URL for selection
	void setVymLinkInt(const QString &);	// Set vymLink for selection
    BranchObj* addNewBranchInt(int);		// pos allows to add above/below selection
public:	
    BranchObj* addNewBranch(int);			// pos allows to add above/below selection
    BranchObj* addNewBranchBefore();		// insert and make selection its
    void deleteSelection();
	LinkableMapObj* getSelection();	// returns selection
	void unselect();				// before changing current noteedit
	void reselect();				// after  changing current noteedit
	bool select(const QString &);	// Select by string
	QString getSelectString();
private:	
	void selectInt(LinkableMapObj*);	
	void selectNextBranchInt();		// Increment number of branch
	void selectPrevBranchInt();		// Decrement number of branch
public:	
    void selectUpperBranch();
    void selectLowerBranch();
    void selectLeftBranch();
    void selectRightBranch();
    void selectFirstBranch();
    void selectLastBranch();
    void selectMapBackgroundImage();
    void setMapBackgroundImage(const QString &);
    void selectMapBackgroundColor();
    void setMapBackgroundColor(QColor);
    QColor getMapBackgroundColor();
    QColor getCurrentHeadingColor();
    void colorBranch(QColor);
    void colorSubtree(QColor);
	void toggleStandardFlag(QString);
    BranchObj* findText(QString,bool);		// Find object
    void findReset();						// Reset Find 
	void setURL(const QString &);			// set the URL
	void editURL();							// edit the URL
	QString getURL();						// returns URL of selection or ""
	QStringList getURLs();					// returns URLs of subtree
	void editHeading2URL();					// copy heading to URL
	void editBugzilla2URL();				// create URL to Bugzilla
	void editFATE2URL();					// create URL to FATE
	void editVymLink();						// edit link to another map
	void deleteVymLink();					// delete link to another map
	QString getVymLink();					// return path to map
	QStringList getVymLinks();				// return paths in subtree
	void setHideExport(bool);				// toggle the export flag
	void toggleHideExport();				// toggle the export flag
	void deleteKeepChilds();				// remove but keep childs
	void deleteChilds();					// remove childs
	void editMapInfo();						// dialog to enter author, ...
	void updateActions();					// update e.g. format buttons
	void updateNoteFlag();					// when TextEditor changes
	void setMapAuthor (const QString &);
	void setMapComment(const QString &);
	void setMapLinkStyle (const QString &);	// Set style of link
	LinkStyle getMapLinkStyle ();			// requested in LMO
	void setMapDefLinkColor(QColor);		// default color of links
	void setMapLinkColorHintInt();			// color of links
	void setMapLinkColorHint(LinkColorHint);// color of links
	LinkColorHint getMapLinkColorHint();
	QColor getMapDefLinkColor();
	void setMapDefXLinkColor(QColor);
	QColor getMapDefXLinkColor();
	void setMapDefXLinkWidth (int);
	int getMapDefXLinkWidth();
	void toggleMapLinkColorHint();			// after changing linkStyles
    void selectMapLinkColor();
	bool scrollBranch();
	bool unscrollBranch();
    void toggleScroll();
    void unscrollChilds();
private:	
	FloatImageObj* loadFloatImageInt (QString);
public:	
	void loadFloatImage ();
private:	
	void saveFloatImageInt (FloatImageObj*, const QString &, const QString &);
public:	
	void saveFloatImage ();
	void setFrame(const FrameType &);
	void setIncludeImagesVer(bool);
	void setIncludeImagesHor(bool);
	void setHideLinkUnselected (bool);
	bool getHideLinkUnselected ();
private:	
    void importDirInt(BranchObj *,QDir);
public:	
    void importDir();
	void followXLink (int);
	void editXLink (int);
    void testFunction();					// just testing new stuff
											// set /mainwindo/showTestMenu=true...

protected:
	void ensureSelectionVisible();		
	virtual void contextMenuEvent ( QContextMenuEvent *e );
    virtual void keyPressEvent(QKeyEvent*);
    virtual void keyReleaseEvent(QKeyEvent*);
    virtual void mousePressEvent(QMouseEvent*);
    virtual void mouseReleaseEvent(QMouseEvent*);
    virtual void mouseDoubleClickEvent(QMouseEvent*);
    virtual void mouseMoveEvent(QMouseEvent*);

	void dragEnterEvent (QDragEnterEvent *);
	void dragMoveEvent (QDragMoveEvent *);
	void dragLeaveEvent (QDragLeaveEvent *);
	void dropEvent (QDropEvent *);

private:
	QGraphicsScene *mapScene;
    MapCenterObj* mapCenter;

	bool adjustCanvasRequested;	// collect requests until end of user event
	BranchObj *editingBO;		// entering Text into BO

	QColor defLinkColor;		// default color for links
	QColor defXLinkColor;		// default color for xlinks
	int defXLinkWidth;			// default width for xlinks
	LinkColorHint linkcolorhint;// use heading color or own color
	LinkStyle linkstyle;		// default style for links

    QCursor HandOpenCursor;		// cursor while moving canvas view
	QCursor PickColorCursor;	// cursor while picking color 
	QCursor CopyCursor;			// cursor while picking color 
	QCursor XLinkCursor;		// cursor while picking color 
	bool pickingColor;
	bool drawingLink;			// true while creating a link
	bool copyingObj;			// true while creating a link
	XLinkObj* tmpXLink;

	Selection xelection;	
    MapObj* movingObj;				// moving a MapObj
	MapObj* linkingObj_src;			// part of a link
    QPointF movingObj_orgPos;		// org. pos of mouse before move
    QPointF movingObj_orgRelPos;	// org. relative pos of mouse before move
    QPointF movingObj_start;		// rel. pos of mouse to absPos 
    QPointF movingCont_start;		// inital pos of moving Content or
    QPointF movingVec;				// how far has Content moved
    QPrinter* printer;				// Printing

    bool mapDefault;				// Flag if map is untouched
    bool mapChanged;				// Flag if undo is possible
	bool mapUnsaved;				// Flag if map should be saved

	Parser parser;				// Parser stuff for scripting

	bool printFrame;			// Print frame around map
	bool printFooter;			// Print footer below map

	bool zipped;				// should map be zipped
	static	int mapNum;			// unique number for Editor
	QString fileName;			// short name of file (for tab)
	QString filePath;			// path to file which will be saved
	QString fileDir;			// dir where file is saved
	QString destPath;			// path to .vym file (needed for vymlinks)
	QString mapName;			// fileName without ".vym"

	QString tmpMapDir;			// tmp directory with undo history
	QString histPath;			// Path to history file
	int stepsTotal;				// total number of steps (undos+redos) 
	SimpleSettings undoSet;		// undo/redo commands
	bool blockReposition;		// block while load or undo
	bool blockSaveState;		// block while load or undo

	BranchObj* itFind;			// next object in find process
	bool EOFind;				// true, if search failed

	QPoint exportOffset;		// set before export, used in save
	HideTmpMode hidemode;			// true while exporting to hide some stuff
	
    void resizeEvent( QResizeEvent * );
};
#endif

